/**************************************************************************
 * FreeDOS 32 FAT Driver                                                  *
 * by Salvo Isaja                                                         *
 *                                                                        *
 * Copyright (C) 2001-2002, Salvatore Isaja                               *
 *                                                                        *
 * This is "rename.c" - Rename/move files and directories                 *
 *                                                                        *
 *                                                                        *
 * This file is part of the FreeDOS 32 FAT Driver.                        *
 *                                                                        *
 * The FreeDOS 32 FAT Driver is free software; you can redistribute it    *
 * and/or modify it under the terms of the GNU General Public License     *
 * as published by the Free Software Foundation; either version 2 of the  *
 * License, or (at your option) any later version.                        *
 *                                                                        *
 * The FreeDOS 32 FAT Driver is distributed in the hope that it will be   *
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 * GNU General Public License for more details.                           *
 *                                                                        *
 * You should have received a copy of the GNU General Public License      *
 * along with the FreeDOS 32 FAT Driver; see the file COPYING;            *
 * if not, write to the Free Software Foundation, Inc.,                   *
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA                *
 **************************************************************************/

#include "fat1.h"
#include "unicode.h"

//#define DEBUG

#ifdef DEBUG
#endif


int fat_rename(void *P, char *OldFullName, char *NewFullName, int UseLfn)
{
  tFatVolume *V = (tFatVolume *) P;
  char        OldPath[FD32_MAX_LFN_PATH_LENGTH];
  char        NewPath[FD32_MAX_LFN_PATH_LENGTH];
  char        OldName[FD32_MAX_LFN_LENGTH];
  char        NewName[FD32_MAX_LFN_LENGTH];
  int         SourceFd;
  int         DestFd;
  tFatFind    D;
  int         Res;

  /* Open the source directory */
  split_path(OldFullName, OldPath, OldName);
  SourceFd = fat_open(V, OldPath,
                      FD32_OPEN_ACCESS_READWRITE | FAT_OPEN_DIRECTORY,
                      FD32_ATTR_NONE, 0, NULL, UseLfn);
  if (SourceFd < 0 ) return SourceFd;

  /* Open the destination directory */
  split_path(NewFullName, NewPath, NewName);
  DestFd = fat_open(V, NewPath,
                    FD32_OPEN_ACCESS_READWRITE | FAT_OPEN_DIRECTORY,
                    FD32_ATTR_NONE, 0, NULL, UseLfn);
  if (DestFd < 0) return DestFd;

  /* Find the source name and get all its informations */
  Res = find(V, SourceFd, OldName,
             FD32_FIND_REQUIRE_NONE | FD32_FIND_ALLOW_ALL, &D, UseLfn);
  if (Res) return Res;

  /* Delete the old entries from the source directory */
  Res = free_dir_entries(V, SourceFd, D.EntryOffset, D.LfnEntries);
  if (Res) return Res;

  /* And allocate the new entries in the destination directory */
  #ifdef FATLFN
  if (UseLfn)
    Res = allocate_lfn_dir_entries(V, DestFd, &D.SfnEntry, NewPath,
                                   NewName, &D.EntryOffset, &D.LfnEntries, 1);
   else
    Res = allocate_sfn_dir_entry(V, DestFd, &D.SfnEntry, NewName, &D.EntryOffset);
  #else
  Res = allocate_sfn_dir_entry(V, DestFd, &D.SfnEntry, NewName, &D.EntryOffset);
  #endif
  
  if (Res)
  {
    /* On failure restore the old entries in the source directory */
//    convert_unicode_string_to_oem(OldName, SourceLongName);
    #ifdef FATLFN
    if (UseLfn)
      allocate_lfn_dir_entries(V, SourceFd, &D.SfnEntry, OldPath,
                               OldName, &D.EntryOffset, &D.LfnEntries, 1);
     else
      allocate_sfn_dir_entry(V, SourceFd, &D.SfnEntry, OldName, &D.EntryOffset);
    #else
    allocate_sfn_dir_entry(V, SourceFd, &D.SfnEntry, OldName, &D.EntryOffset);
    #endif
  }
  fat_close(V, SourceFd);
  fat_close(V, DestFd);
  return Res;
}
