/**************************************************************************
 * FreeDOS 32 Unicode support library                                     *
 * by Salvo Isaja                                                         *
 *                                                                        *
 * Copyright (C) 2001, Salvatore Isaja                                    *
 * This file contains parts based on the DJGPP's libc version 2.03,       *
 * Copyright (C) DJ Delorie, http://www.delorie.com/djgpp/                *
 *                                                                        *
 * This file is part of FreeDOS 32                                        *
 *                                                                        *
 * FreeDOS 32 is free software; you can redistribute it and/or modify it  *
 * under the terms of the GNU General Public License as published by the  *
 * Free Software Foundation; either version 2 of the License, or (at your *
 * option) any later version.                                             *
 *                                                                        *
 * FreeDOS 32 is distributed in the hope that it will be useful, but      *
 * WITHOUT ANY WARRANTY; without even the implied warranty of             *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 * GNU General Public License for more details.                           *
 *                                                                        *
 * You should have received a copy of the GNU General Public License      *
 * along with the FreeDOS 32 FAT Driver; see the file COPYING;            *
 * if not, write to the Free Software Foundation, Inc.,                   *
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA                *
 **************************************************************************/

#include "unicode.h"

/* The following codes have been extracted from the article:
 * "NOTES ON THE STRUCTURE OF THE VFAT FILESYSTEM", VFATInfo.txt,
 * by Galen C. Hunt <gchunt@cs.rochester.edu>, Gordon Chaffee,
 * from the table "NOTES ON UNICODE TRANSLATION IN VFAT FILESYSTEM"
 * by Steve Searle <steve@mgu.bath.ac.uk>.
 *
 * Thanks to RHIDE macros which helped a lot in writing this in C! :-)
 */
 
/* Oem -> Unicode mapping for characters 128..255 */
static unsigned short OemToUnicode[128] =
{
  0x00C7, 0x00FC, 0x00E9, 0x00E2, 0x00E4, 0x00E0, 0x00E5, 0x00E7,
  0x00EA, 0x00EB, 0x00E8, 0x00EF, 0x00EE, 0x00EC, 0x00C4, 0x00C5,
  0x00C9, 0x00E6, 0x00C6, 0x00F4, 0x00F6, 0x00F2, 0x00FB, 0x00F9,
  0x00FF, 0x00D6, 0x00DC, 0x00F8, 0x00A3, 0x00D8, 0x00D7, 0x0092,
  0x00E1, 0x00E0, 0x00F3, 0x00FA, 0x00F1, 0x00D1, 0x00AA, 0x00BA,
  0x00BF, 0x00AE, 0x00AC, 0x00BD, 0x00BC, 0x00A1, 0x00AB, 0x00BB,
  0x2591, 0x2592, 0x2593, 0x2502, 0x2524, 0x00C1, 0x00C2, 0x00C0,
  0x00A9, 0x2563, 0x2551, 0x2557, 0x255D, 0x00A2, 0x00A5, 0x2510,
  0x2514, 0x2534, 0x252C, 0x251C, 0x2500, 0x253C, 0x00E3, 0x00C3,
  0x255A, 0x2554, 0x2569, 0x2566, 0x2560, 0x2550, 0x256C, 0x00A4,
  0x00F0, 0x00D0, 0x00CA, 0x00CB, 0x00C8, 0x0131, 0x00CD, 0x00CE,
  0x00CF, 0x2518, 0x250C, 0x2588, 0x2584, 0x00A6, 0x00CC, 0x2580,
  0x00D3, 0x00DF, 0x00D4, 0x00D2, 0x00F5, 0x00D5, 0x00B5, 0x00FE,
  0x00DE, 0x00DA, 0x00DB, 0x00D9, 0x00FD, 0x00DD, 0x00AF, 0x00B4,
  0x00AD, 0x00B1, 0x2517, 0x00BE, 0x00B6, 0x00A7, 0x00F7, 0x00B8,
  0x00B0, 0x00A8, 0x00B7, 0x00B9, 0x00B3, 0x00B2, 0x25A0, 0x00A0
};


/* Converts between Unicode scalar value and UTF-16 character of a string.
 *
 *           Scalar                              UTF-16
 * 00000000 zzzzyyyy yyxxxxxx <-> zzzzyyyy yyxxxxxx
 * 000uuuuu zzzzyyyy yyxxxxxx <-> 110110ww wwzzzzyy  110111yy yyxxxxxx
 * where wwww = uuuuu - 1.
 */
int utf16_to_scalar(WORD **s, DWORD *Ch)
{
  if ((**s & 0xFC00) != 0xD800)
  {
    *Ch = *(*s)++;
    return 0;
  }
  *Ch = ((*(*s)++ & 0x03FF) << 10) + 0x010000;
  if ((**s & 0xFC00) != 0xDC00) return 1; /* Invalid UTF-16 sequence */
  *Ch += *(*s)++ & 0x03FF;
  return 0;
}


int scalar_to_utf16(DWORD Ch, WORD **s)
{
  if (Ch < 0x010000)
  {
    *(*s)++ = (WORD) Ch;
    return 0;
  }
  if (Ch < 0x200000)
  {
    *(*s)++ = 0xD800 + (((Ch >> 16) - 1) << 6) + ((Ch & 0x00FC00) >> 2);
    *(*s)++ = 0xDC00 + (Ch & 0x0003FF);
    return 0;
  }
  return 1; /* Invalid scalar value */
}


/* Converts between Unicode scalar value and UTF-8 character of a string.
 *
 *           Scalar                               UTF-8
 * 00000000 00000000 0xxxxxxx <-> 0xxxxxxx
 * 00000000 00000yyy yyxxxxxx <-> 110yyyyy  10xxxxxx
 * 00000000 zzzzyyyy yyxxxxxx <-> 1110zzzz  10yyyyyy  10xxxxxx
 * 000uuuuu zzzzyyyy yyxxxxxx <-> 11110uuu  10uuzzzz  10yyyyyy  10xxxxxx
 */
int utf8_to_scalar(char **s, DWORD *Ch)
{
  if (!(**s & 0x80))
  {
    *Ch = *(*s)++;
    return 0;
  }
  if ((**s & 0xE0) == 0xC0)
  {
    *Ch = (*(*s)++ & 0x8F) << 6;
    if ((**s & 0xC0) != 0x80) return 1; /* Invalid UTF-8 sequence */
    *Ch += *(*s)++ & 0x3F;
    return 0;
  }
  if ((**s & 0xF0) == 0xE0)
  {
    *Ch = (*(*s)++ & 0x0F) << 12;
    if ((**s & 0xC0) != 0x80) return 1; /* Invalid UTF-8 sequence */
    *Ch += (*(*s)++ & 0x3F) << 6;
    if ((**s & 0xC0) != 0x80) return 1; /* Invalid UTF-8 sequence */
    *Ch += *(*s)++ & 0x3F;
    return 0;
  }
  if ((**s & 0xF8) == 0xF0)
  {
    *Ch = (*(*s)++ & 0x07) << 18;
    if ((**s & 0xC0) != 0x80) return 1; /* Invalid UTF-8 sequence */
    *Ch = (*(*s)++ & 0x3F) << 12;
    if ((**s & 0xC0) != 0x80) return 1; /* Invalid UTF-8 sequence */
    *Ch += (*(*s)++ & 0x3F) << 6;
    if ((**s & 0xC0) != 0x80) return 1; /* Invalid UTF-8 sequence */
    *Ch += *(*s)++ & 0x3F;
    return 0;
  }
  ++*s;
  return 1; /* Invalid UTF-8 sequence */
}


int scalar_to_utf8(DWORD Ch, char **s)
{
  if (Ch < 0x000080)
  {
    *(*s)++ = (char) Ch;
    return 0;
  }
  if (Ch < 0x000800)
  {
    *(*s)++ = 0xC0 + ((Ch & 0x0007C0) >> 6);
    *(*s)++ = 0x80 +  (Ch & 0x00003F);
    return 0;
  }
  if (Ch < 0x010000)
  {
    *(*s)++ = 0xE0 + ((Ch & 0x00F000) >> 12);
    *(*s)++ = 0x80 + ((Ch & 0x000FC0) >> 6);
    *(*s)++ = 0x80 +  (Ch & 0x00003F);
    return 0;
  }
  if (Ch < 0x200000)
  {
    *(*s)++ = 0xF0 + ((Ch & 0xFC0000) >> 18);
    *(*s)++ = 0x80 + ((Ch & 0x03F000) >> 12);
    *(*s)++ = 0x80 + ((Ch & 0x000FC0) >> 6);
    *(*s)++ = 0x80 +  (Ch & 0x00003F);
    return 0;
  }
  return 1; /* Invalid scalar value */
}


/* Converts a UTF-8 string into UTF-16 */
int utf8_to_utf16(char *Utf8, WORD *Utf16)
{
  DWORD Scalar;
  do
  {
    if (utf8_to_scalar(&Utf8, &Scalar)) return 1;
    if (scalar_to_utf16(Scalar, &Utf16)) return 1;
  }
  while (Scalar != 0x000000);
  return 0;
}


/* Converts a UTF-16 string into UTF-8 */
int utf16_to_utf8(WORD *Utf16, char *Utf8)
{
  DWORD Scalar;
  do
  {
    if (utf16_to_scalar(&Utf16, &Scalar)) return 1;
    if (scalar_to_utf8(Scalar, &Utf8)) return 1;
  }
  while (Scalar != 0x000000);
  return 0;
}


/* Converts the ASCII character Ch of the OEM code page in Unicode */
int convert_oem_char_to_unicode(unsigned char Ch)
{
  if (Ch < 128) return Ch;
  return OemToUnicode[Ch - 128];
}


/* Converts the Unicode character Ch in ASCII with OEM code page */
unsigned char convert_unicode_char_to_oem(int Ch)
{
  int k;
  if (Ch < 128) return Ch;
  for (k=0;k<128;k++)
    if (OemToUnicode[k] == Ch) return k + 128;
  return '_'; /* Unconvertable character */
}


/* Copies the ASCII OEM string From into the string To, converting it
 * to Unicode.
 *
 * Returns a pointer to the converted string.
 *
 * This function has been extracted from the DJGPP's libc.
 * Package: djlsr203.zip, File: src\libc\ansi\string\strcpy.c
 */
/* Copyright (C) 1994 DJ Delorie, see COPYING.DJ for details */
unsigned short *convert_oem_string_to_unicode(unsigned short *To,
                                              const unsigned char *From)
{
  unsigned short *Save = To;

  for (; (*To = convert_oem_char_to_unicode(*From)); ++From, ++To);
  return Save;
}


/* Copies the Unicode string From into the string To, converting it
 * to ASCII OEM.
 *
 * Returns a pointer to the converted string.
 *
 * This function has been extracted from the DJGPP's libc.
 * Package: djlsr203.zip, File: src\libc\ansi\string\strcpy.c
 */
/* Copyright (C) 1994 DJ Delorie, see COPYING.DJ for details */
unsigned char *convert_unicode_string_to_oem(unsigned char *To,
                                             const unsigned short *From)
{
  unsigned char *Save = To;

  for (; (*To = convert_unicode_char_to_oem(*From)); ++From, ++To);
  return Save;
}


/* Returns the Unicode Ch converted to upper case if it is lower case.
 *
 * NOTE: Works only on characters in range [0x0000..0x00FF] plus any
 *       character used to map ASCII into Unicode in Long File Names.
 */
int unicode_toupper(int Ch)
{
  switch(Ch)
  {
    /* MICRO SIGN */
    case 0x00B5            : return 0x039C;
    /* LATIN SMALL LETTER Y WITH DIAERESIS */
    case 0x00FF            : return 0x0178;
    /* LATIN SMALL LETTER DOTLESS I */
    case 0x0131            : return 0x0049;
    /* All others unchanged */
    default                :
      if( (Ch >= 0x0061) && (Ch <= 0x007a) )
        return Ch + 0x0041 - 0x0061;
      else if( (Ch >= 0x00e0) && (Ch <= 0x00f6) )
        return Ch + 0x00C0 - 0x00E0;
      else if( (Ch >= 0x00f8) && (Ch <= 0x00fe) )
        return Ch + 0x00D8 - 0x00F8;
      else
        return Ch;
  }
}


/* This function compares S1 and S2.
 *
 * Returns Zero if the strings are equal, a positive number if S1 comes
 * after S2 in the ASCII collating sequense, else a negative number.
 *
 * This function has been extracted from the DJGPP's libc.
 * Package: djlsr203.zip, File: src\libc\ansi\string\strcmp.c
 */
/* Copyright (C) 1994 DJ Delorie, see COPYING.DJ for details          */
int unicode_strcmp(const unsigned short *s1, const unsigned short *s2)
{
  while (*s1 == *s2)
  {
    if (*s1 == 0) return 0;
    s1++;
    s2++;
  }
  return *(unsigned const short *)s1 - *(unsigned const short *)(s2);
}


/* This function compares the two strings, disregarding case.
 *
 * Returns Zero if they're the same, nonzero if different, the sign indicates
 * "order".
 *
 * This function has been extracted from the DJGPP's libc.
 * Package: djlsr203.zip, File: src\libc\compat\string\stricmp.c
 */
/* Copyright (C) 1999 DJ Delorie, see COPYING.DJ for details */
/* Copyright (C) 1995 DJ Delorie, see COPYING.DJ for details */
int unicode_stricmp(const unsigned short *s1, const unsigned short *s2)
{
  while (unicode_toupper(*s1) == unicode_toupper(*s2))
  {
    if (*s1 == 0) return 0;
    s1++;
    s2++;
  }
  return (int)unicode_toupper(*s1) - (int)unicode_toupper(*s2);
}


/* This function copies From into To.
 * Returns a pointer to To.
 *
 * This function has been extracted from the DJGPP's libc.
 * Package: djlsr203.zip, File: src\libc\ansi\string\strcpy.c
 */
/* Copyright (C) 1994 DJ Delorie, see COPYING.DJ for details */
unsigned short *unicode_strcpy(unsigned short *To, const unsigned short *From)
{
  unsigned short *Save = To;

  for (; (*To = *From); ++From, ++To);
  return Save;
}
