/****************************************************************************
*
* (C) 2000 by BECK IPC GmbH
*
*  BECK IPC GmbH
*  Garbenheimerstr. 38
*  D-35578 Wetzlar
*
*  Phone : (49)-6441-905-240
*  Fax   : (49)-6441-905-245
*
* IPC@CHIP SC12 API 
* ---------------------------------------------------------------------------
* $Header: C:\proj2001\IPC@CHIP\Examples\CLIB\HWAPI.C, 3, 19.06.01 13:35:06, Markus Bartat$
* ---------------------------------------------------------------------------
* $Log: 
*  4    IPC@CHIP  1.4         02.08.01 20:13:00    Thomas Kinder 
*       Added an explicit cast in hal_install_isr() to make it cpp compileable.
*       Implemented all missing functions for BIOS v1.0:
*         - hal_init_nv_data(_REMOP remop);
*         - hal_save_nv_data();
*         - hal_get_reboot_reason();
*         - hal_get_frequency(unsigned char which);
*         - hal_set_timer_duty(unsigned char timer,
*         - pfe_set_int_mode (unsigned intr, unsigned char mode);
*         - pfe_enable_pwd();
*  3    IPC@CHIP  1.2         19.06.01 13:35:06    Markus Bartat   start: Added
*       comments to the c functions
*  2    IPC@CHIP  1.1         31.05.01 16:18:23    Christoph Stoidner 
*  1    IPC@CHIP  1.0         14.02.01 16:09:18    Christoph Stoidner 
* $
*****************************************************************************/

/***************************************************************************/
// Includes
/***************************************************************************/
#include <dos.h>
#include "hwapi.h"

/***************************************************************************/
// Defines
/***************************************************************************/

/***************************************************************************/
// Functions
/***************************************************************************/

/***************************************************************************/
// Enable bus
/***************************************************************************/
void pfe_enable_bus (unsigned mask, unsigned char ale)
{
  // Enable bus mask
  asm mov ah, 0x80
  asm mov al, ale
  asm mov dx, mask
  asm int PFE_INT
}

/***************************************************************************/
// Enable non multplexed address bus
/***************************************************************************/
void pfe_enable_adr (unsigned char adr)
{
  // Enable non multiplexed io-bus
  asm mov ah, 0x81
  asm mov dx, 1
  asm mov cl, adr
  asm shl dx, cl    ;// dx = 1<<adr
  asm int PFE_INT
}

/***************************************************************************/
// Enable programmable chip selects
/***************************************************************************/
void pfe_enable_pcs (unsigned char pcs)
{
  // Enable programmable chip selects
  asm mov ah, 0x83
  asm mov dx, 1
  asm mov cl, pcs
  asm shl dx, cl    ;// dx = 1<<pcs
  asm int PFE_INT
}

/***************************************************************************/
// Enable interrupts
/***************************************************************************/
void pfe_enable_int (unsigned char irq)
{
  // Enable interrupts
  asm mov ah, 0x84
  asm mov dx, 1
  asm mov cl, irq
  asm shl dx, cl    ;// dx = 1<<irq
  asm int PFE_INT
}

/***************************************************************************/
// Enable timer
/***************************************************************************/
void pfe_enable_timer (unsigned mode)
{
  // Enable timer
  asm mov ah, 0x85
  asm mov dx, mode
  asm int PFE_INT
}

/***************************************************************************/
// Enable programmable pio pins
/***************************************************************************/
void pfe_enable_pio (unsigned pio, unsigned char mode)
{
  // Enable programmable pios
  asm mov ah, 0x82
  asm mov al, mode

  asm mov dx, 1
  asm mov cx, pio
  asm shl dx, cl    ;// dx = 1<<pio

  asm int PFE_INT
}

/***************************************************************************/
// Get programmable pio pins state
/***************************************************************************/
void pfe_get_pio_state (unsigned *wPio, unsigned *wInp, unsigned *wOut)
{
  unsigned p, i, o;

  asm mov ah, 0x82
  asm mov al, 0
  asm int PFE_INT
  asm mov p, ax
  asm mov i, dx
  asm mov o, cx

  *wPio = p;
  *wInp = i;
  *wOut = o;
}

/***************************************************************************/
// Get programmable pio pins output state
/***************************************************************************/
void hal_get_pio_output_state (unsigned *pins, unsigned wOut)
{
  unsigned p;

  // Read pio outputs init state
  asm mov ah, 0x82
  asm mov bx, 0xFFFF
  asm xor cx, cx        ;// cx = 0
  asm int HAL_INT
  asm and ax, wOut
  asm mov p, ax

  *pins = p; //*pins = ax & wOut;
}

/***************************************************************************/
// Set Edge/Level Interrupt Mode
/***************************************************************************/
void  pfe_set_int_mode (unsigned intr, unsigned char mode)
{
  // Set Edge/Level Interrupt Mode
  asm mov ah, 0x86
  asm mov al, mode
  asm mov dx, 1
  asm mov cl, intr
  asm shl dx, cl    ;// dx = 1<<intr
  asm int PFE_INT
}

/***************************************************************************/
// Enable PWD Mode
/***************************************************************************/
void  pfe_enable_pwd()
{
  // Enable PWD Mode
  asm mov ah, 0x87
  asm int PFE_INT
}

/***************************************************************************/
// Read databus
/***************************************************************************/
unsigned char hal_read_bus (unsigned adr, unsigned wAND, unsigned wXOR)
{
  unsigned char retval;

  // Read databus
  asm mov ah, 0x80
  asm mov di, adr
  asm mov bx, wAND
  asm mov cx, wXOR
  asm int HAL_INT
  asm mov retval, al

  return retval;
}

/***************************************************************************/
// Write databus
/***************************************************************************/
void hal_write_bus (unsigned adr, unsigned char value, unsigned wAND, unsigned wXOR)
{
  // Write databus
  asm mov ah, 0x81
  asm mov di, adr
  asm mov dh, 0
  asm mov dl, value
  asm mov bx, wAND
  asm mov cx, wXOR
  asm int HAL_INT
}

/***************************************************************************/
// Read block from databus
/***************************************************************************/
void hal_readblock_bus (unsigned adr0, unsigned adr1, unsigned char *buf, unsigned cnt)
{
  // read databus
  _ES = FP_SEG(buf);
  _BX = FP_OFF(buf);
  asm mov ah, 0xA0
  asm mov di, adr0
  asm mov si, adr1
  asm mov cx, cnt
  asm int HAL_INT
}

/***************************************************************************/
// Write block to databus
/***************************************************************************/
void hal_writeblock_bus (unsigned adr0, unsigned adr1, unsigned char *buf, unsigned cnt)
{
  // write databus
  _ES = FP_SEG(buf);
  _BX = FP_OFF(buf);
  asm mov ah, 0xA1
  asm mov di, adr0
  asm mov si, adr1
  asm mov cx, cnt
  asm int HAL_INT
}

/***************************************************************************/
// Read pios
/***************************************************************************/
unsigned hal_read_pios (unsigned wAND, unsigned wXOR)
{
  unsigned retval;

  // Read pios
  asm mov ah, 0x82
  asm mov bx, wAND
  asm mov cx, wXOR
  asm int HAL_INT
  asm mov retval, ax

  return retval;
}

/***************************************************************************/
// Write pios
/***************************************************************************/
void hal_write_pios (unsigned value, unsigned wAND, unsigned wXOR)
{
  // Write pios
  asm mov ah, 0x83
  asm mov dx, value
  asm mov bx, wAND
  asm mov cx, wXOR
  asm int HAL_INT
}

/***************************************************************************/
// Read single pio
/***************************************************************************/
unsigned char hal_read_pio (unsigned char pio)
{
  unsigned retval;

  asm mov ah, 0x8C
  asm mov al, pio
  asm int HAL_INT
  asm mov retval, ax

  if (retval) return 1;
  else        return 0;
}

/***************************************************************************/
// Write single pio
/***************************************************************************/
void hal_write_pio (unsigned char pio, unsigned value)
{
  asm mov ah, 0x8D
  asm mov al, pio
  asm mov dx, value
  asm int HAL_INT
}

/***************************************************************************/
// Install ISR
/***************************************************************************/
InterruptHandler hal_install_isr (unsigned irq, unsigned count, InterruptHandler handler)
{
  InterruptHandler oldhandler;
  unsigned rbx, res;

  // install isr
  asm les bx, handler  ;  // es = FP_SEG(handler);  bx = FP_OFF(handler);
  asm mov ah, 0x84
  asm mov dx, irq
  asm mov cx, count
  asm int HAL_INT
  asm mov rbx, bx
  asm mov res, es

  oldhandler = (InterruptHandler)MK_FP(res,rbx);

  return oldhandler;
}



/***************************************************************************/
// Get frequencies
/***************************************************************************/
unsigned long hal_get_frequency(unsigned char which)
{
   unsigned long freq;

   asm mov ah, 0x8A
   asm mov al, which
   asm int HAL_INT
   asm mov [freq+0], ax
   asm mov [freq+2], dx

   return freq;
}


/***************************************************************************/
// Init timer
/***************************************************************************/
void hal_init_timer (unsigned char timer, unsigned mode, unsigned divider)
{
  // init timer
  asm mov ah, 0x85
  asm mov al, timer
  asm mov dx, mode
  asm mov cx, divider
  asm int HAL_INT
}


/***************************************************************************/
// Set Timer Duty Cycle Waveform
/***************************************************************************/
void hal_set_timer_duty (unsigned char timer, unsigned mode, unsigned divider)
{
  // set duty cycle
  asm mov ah, 0x8B
  asm mov al, timer
  asm mov dx, mode
  asm mov cx, divider
  asm int HAL_INT
}


/***************************************************************************/
// Start timer
/***************************************************************************/
void hal_start_timer (unsigned char timer)
{
  // start timer
  asm mov ah, 0x86
  asm mov al, timer
  asm int HAL_INT
}

/***************************************************************************/
// Stop timer
/***************************************************************************/
void hal_stop_timer (unsigned char timer)
{
  // stop timer
  asm mov ah, 0x87
  asm mov al, timer
  asm int HAL_INT
}

/***************************************************************************/
// Read timer count
/***************************************************************************/
unsigned hal_read_timer_count (unsigned char timer, unsigned far *maxcount)
{
  unsigned retval, count;

  // read count
  asm mov ah, 0x88
  asm mov al, timer
  asm int HAL_INT
  asm mov count, dx
  asm mov retval, ax

  *maxcount = count;
  return retval;
}

/***************************************************************************/
// Write timer count
/***************************************************************************/
void hal_write_timer_count (unsigned char timer, unsigned counter)
{
  // write count
  asm mov ah, 0x89
  asm mov al, timer
  asm mov dx, counter
  asm int HAL_INT
}

/***************************************************************************/
// Set watchdog mode
/***************************************************************************/
unsigned char hal_set_watchdog_mode (unsigned char mode)
{
  unsigned char retval;

  // set watchdog mode
  asm mov ah, 0x90
  asm mov al, mode
  asm int HAL_INT
  asm mov retval, al

  return retval;
}

/***************************************************************************/
// Refresh watchdog
/***************************************************************************/
void hal_refresh_watchdog (void)
{
  // refresh watchdog
  asm mov ah, 0x91
  asm int HAL_INT
}




/***************************************************************************/
// Initialize/Restore Non-Volatile Data
/***************************************************************************/
unsigned hal_init_nv_data(_REMOP remop)
{
  unsigned retval;
  
  asm les bx, remop  ;  // es = FP_SEG(remop);  bx = FP_OFF(remop);
  asm mov ah, 0xC0
  asm int HAL_INT
  asm mov retval, ax

  return retval;
}


/***************************************************************************/
// Save Non-Volatile Data
/***************************************************************************/
void hal_save_nv_data()
{
  // save data
  asm mov ah, 0xC1
  asm int HAL_INT
}


/***************************************************************************/
// Get Reboot Reason
/***************************************************************************/
unsigned hal_get_reboot_reason()
{
  unsigned reason;

  // get reason
  asm mov ah, 0xC2
  asm int HAL_INT
  asm mov reason, ax

  return reason;
}
